weblab

package
v0.0.0-...-30fea7a Latest Latest
Warning

This package is not in the latest version of its module.

Go to latest
Published: Aug 21, 2026 License: BSD-3-Clause Imports: 4 Imported by: 1

Documentation

Overview

Package weblab extends the fgbase package with web and http primitives.

Index

Constants

This section is empty.

Variables

This section is empty.

Functions

func FuncHTTP

func FuncHTTP(x fgbase.Edge, addr string, quitChan chan struct{}) fgbase.Node

FuncHTTP creates an http server and passes requests downstream.

func FuncKcons

func FuncKcons(x fgbase.Edge, topic string) fgbase.Node

FuncKcons wraps a Kafka consumer.

func FuncKprod

func FuncKprod(a fgbase.Edge) fgbase.Node

FuncKprod wraps a Kafka producer.

Types

This section is empty.

Jump to